#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import pytest
import numpy as np
from numpy.testing import assert_equal, assert_allclose
from astropy.utils.misc import NumpyRNGContext
try:
    import healpy as hp
    DO_HEALPY = True
except ImportError:
    DO_HEALPY = False

from cygrid import Healpix


# for order in range(0, 12):
#     nside = 2 ** order
#     hpx = cygrid.Healpix(nside)
#     print hpx.nside, hpx.npface, hpx.npix, hpx.nrings, hpx.omega, hpx.order, hpx.resolution

hpx_props = [
    (1, 1, 12, 3, 1.0471975512, 0, 1.02332670795),
    (2, 4, 48, 7, 0.261799387799, 1, 0.511663353973),
    (4, 16, 192, 15, 0.0654498469498, 2, 0.255831676987),
    (8, 64, 768, 31, 0.0163624617374, 3, 0.127915838493),
    (16, 256, 3072, 63, 0.00409061543436, 4, 0.0639579192467),
    (32, 1024, 12288, 127, 0.00102265385859, 5, 0.0319789596233),
    (64, 4096, 49152, 255, 0.000255663464648, 6, 0.0159894798117),
    (128, 16384, 196608, 511, 6.39158661619e-05, 7, 0.00799473990583),
    (256, 65536, 786432, 1023, 1.59789665405e-05, 8, 0.00399736995292),
    (512, 262144, 3145728, 2047, 3.99474163512e-06, 9, 0.00199868497646),
    (1024, 1048576, 12582912, 4095, 9.9868540878e-07, 10, 0.000999342488229),
    (2048, 4194304, 50331648, 8191, 2.49671352195e-07, 11, 0.000499671244114),
    ]


class TestHealpix:

    def setup(self):

        pass
        # self.nside = 1024
        # self.hpx = Healpix(self.nside)

    def teardown(self):

        pass

    def test_properties(self):

        eps = 1.e-10

        for nside, npface, npix, nrings, omega, order, reso in hpx_props:

            hpx = Healpix(nside)

            assert hpx.nside == nside
            assert hpx.npface == npface
            assert hpx.npix == npix
            assert hpx.nrings == nrings
            assert abs(hpx.omega - omega) < eps
            assert hpx.order == order
            assert abs(hpx.resolution - reso) < eps

    def test_get_ring_info_small(self):

        hpx = Healpix(64)

        assert hpx.get_ring_info_small(1) == (0, 4, True)
        assert hpx.get_ring_info_small(10) == (180, 40, True)
        assert hpx.get_ring_info_small(127) == (24192, 256, False)
        assert hpx.get_ring_info_small(128) == (24448, 256, True)
        assert hpx.get_ring_info_small(255) == (49148, 4, True)

        with pytest.raises(ValueError):
            hpx.get_ring_info_small(0)
        with pytest.raises(ValueError):
            hpx.get_ring_info_small(256)

    def test_pix2ring(self):

        if not DO_HEALPY:
            # not possible on Windows
            return

        with NumpyRNGContext(12345):

            nside = 1024
            hpx = Healpix(nside)
            npix = hpx.npix

            hpx_idx = np.random.random_integers(0, npix - 1, 10000)

            assert_equal(
                hp.pix2ring(nside, hpx_idx),
                hpx.pix2ring_many(hpx_idx)
                )

    def test_ang2pix(self):

        if not DO_HEALPY:
            # not possible on Windows
            return

        with NumpyRNGContext(12345):

            nside = 1024
            hpx = Healpix(nside)

            theta = np.random.uniform(0, np.pi, 10000)
            phi = np.random.uniform(0, 2 * np.pi, 10000)

            assert_equal(
                hp.ang2pix(nside, theta, phi),
                hpx.ang2pix_many(theta, phi)
                )

    def test_pix2ang(self):

        if not DO_HEALPY:
            # not possible on Windows
            return

        with NumpyRNGContext(12345):

            nside = 1024
            hpx = Healpix(nside)
            npix = hpx.npix

            hpx_idx = np.random.random_integers(0, npix - 1, 10000)

            assert_allclose(
                hp.pix2ang(nside, hpx_idx),
                hpx.pix2ang_many(hpx_idx)
                )

    def test_query_disc(self):

        if not DO_HEALPY:
            # not possible on Windows
            return

        with NumpyRNGContext(12345):

            nside = 1024
            hpx = Healpix(nside)
            disc_size = np.radians(0.5 / 60.)

            thetas = np.random.uniform(0, np.pi, 10)
            phis = np.random.uniform(0, 2 * np.pi, 10)

            for theta, phi in zip(thetas, phis):

                _vec = hp.ang2vec(theta, phi)
                hp_disc = hp.query_disc(nside, _vec, disc_size)
                assert_equal(
                    hp_disc,
                    hpx.query_disc(theta, phi, disc_size)
                    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import pytest
import numpy as np
from numpy.testing import assert_allclose
from astropy.utils.misc import NumpyRNGContext
from astropy.utils.data import get_pkg_data_filename
import cygrid


class TestWcsGrid:

    def setup(self):

        mapcenter = (131., 50.)
        mapsize = (1., 1.)  # degrees
        self.beamsize_fwhm = 0.05  # degrees

        avg_num_pixels_x = 5 * mapsize[0] / self.beamsize_fwhm
        avg_num_pixels_y = 5 * mapsize[1] / self.beamsize_fwhm

        scale = np.cos(np.radians(mapcenter[1]))

        with NumpyRNGContext(0):
            self.xcoords = np.random.uniform(
                mapcenter[0] - mapsize[0] / 2. / scale,
                mapcenter[0] + mapsize[0] / 2. / scale,
                int(avg_num_pixels_x * avg_num_pixels_y),
                ).astype(np.float64)
            self.ycoords = np.random.uniform(
                mapcenter[1] - mapsize[1] / 2.,
                mapcenter[1] + mapsize[1] / 2.,
                int(avg_num_pixels_x * avg_num_pixels_y),
                ).astype(np.float64)
            self.signal = np.random.normal(0., .01, len(self.xcoords))

        self.signal2 = np.column_stack([self.signal, self.signal ** 2])
        self.signal3 = np.column_stack([
            self.signal, self.signal ** 2,
            self.signal ** 3, self.signal ** 4
            ]).reshape((-1, 2, 2))
        pixsize = self.beamsize_fwhm / 3.
        dnaxis1 = int(mapsize[0] / pixsize + 0.5)
        dnaxis2 = int(mapsize[1] / pixsize + 0.5)
        self.yx_shape = dnaxis2, dnaxis1
        projection = 'SIN'

        self.header = {
            'NAXIS': 2,
            'NAXIS1': dnaxis1,
            'NAXIS2': dnaxis2,
            'CTYPE1': 'RA---{}'.format(projection),
            'CTYPE2': 'DEC--{}'.format(projection),
            'CUNIT1': 'deg',
            'CUNIT2': 'deg',
            'CDELT1': -pixsize,
            'CDELT2': pixsize,
            'CRPIX1': dnaxis1 / 2.,
            'CRPIX2': dnaxis2 / 2.,
            'CRVAL1': mapcenter[0],
            'CRVAL2': mapcenter[1],
            }

        kernelsize_fwhm = self.beamsize_fwhm / 2
        kernelsize_sigma = kernelsize_fwhm / 2.35
        support_radius = 3. * kernelsize_sigma
        hpx_min_res = kernelsize_sigma / 2.

        self.kernel_args = (
            'gauss1d',
            (kernelsize_sigma, ),
            support_radius,
            hpx_min_res,
            )

        self.test_maps = np.load(get_pkg_data_filename(
            'data/cygrid_test_maps.npy'
            ))

    def teardown(self):

        pass

    def test_gridding2d(self):

        mygridder = cygrid.WcsGrid(self.header)
        mygridder.set_kernel(*self.kernel_args)

        mygridder.grid(self.xcoords, self.ycoords, self.signal)

        assert_allclose(
            mygridder.get_datacube(),
            self.test_maps[0, 0],
            atol=1.e-6
            )

    def test_gridding3d(self):

        mygridder = cygrid.WcsGrid(self.header)
        mygridder.set_kernel(*self.kernel_args)

        mygridder.grid(self.xcoords, self.ycoords, self.signal2)

        assert_allclose(
            mygridder.get_datacube(),
            self.test_maps[0],
            atol=1.e-6
            )

    def test_gridding4d(self):

        mygridder = cygrid.WcsGrid(self.header)
        mygridder.set_kernel(*self.kernel_args)

        mygridder.grid(self.xcoords, self.ycoords, self.signal3)

        if False:
            np.save('/tmp/cygrid_test_maps.npy', mygridder.get_datacube())

        assert_allclose(
            mygridder.get_datacube(),
            self.test_maps,
            atol=1.e-6
            )

    def test_kernel_not_set_error(self):

        mygridder = cygrid.WcsGrid(self.header)

        with pytest.raises(RuntimeError):
            mygridder.grid(self.xcoords, self.ycoords, self.signal)

    def test_shape_error(self):
        '''
        Test for various shape errors
        '''

        mygridder = cygrid.WcsGrid(self.header)
        mygridder.set_kernel(*self.kernel_args)

        with pytest.raises(cygrid.ShapeError):
            mygridder.grid(
                self.xcoords, self.ycoords[:, np.newaxis], self.signal
                )

        with pytest.raises(cygrid.ShapeError):
            # this should not work, as the second call to grid has
            # incompatible shape
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal
                )
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal[:, np.newaxis]
                )

        # now test with user-provided data cube
        dcube = np.zeros_like(self.test_maps[0, 0])
        mygridder = cygrid.WcsGrid(self.header, datacube=dcube)
        mygridder.set_kernel(*self.kernel_args)

        with pytest.raises(cygrid.ShapeError):
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal[:, np.newaxis]
                )

    def test_dtype_warning(self):
        '''
        Test for dtype warning

        In fact, at the moment there is only one possible warning, as
        everything else should be properly casted, etc.
        '''

        dcube = np.zeros_like(self.test_maps[0, 0])  # float32
        mygridder = cygrid.WcsGrid(
            self.header, datacube=dcube, dtype=np.float64
            )
        mygridder.set_kernel(*self.kernel_args)

        with pytest.warns(UserWarning):
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal
                )

    def test_byteorder_warning(self):
        '''
        Test for byteorder warning

        Apparently astroquery.skyview can return wrong (non-native) byteorder.
        '''

        dcube = np.zeros_like(self.test_maps[0, 0])  # float32
        mygridder = cygrid.WcsGrid(
            self.header, datacube=dcube, dtype=np.float64
            )
        mygridder.set_kernel(*self.kernel_args)

        signal_swapped = self.signal.byteswap().newbyteorder()

        with pytest.warns(UserWarning):
            mygridder.grid(
                self.xcoords, self.ycoords, signal_swapped
                )

    def test_c_contiguous(self):
        '''
        Cygrid should autocast to C-contiguous if necessary and raise
        an error if user-provided datacube is not C-contiguous
        '''

        signal2_f_cont = np.require(self.signal2, self.signal2.dtype, 'F')

        mygridder = cygrid.WcsGrid(self.header)
        mygridder.set_kernel(*self.kernel_args)

        mygridder.grid(self.xcoords, self.ycoords, signal2_f_cont)

        assert_allclose(
            mygridder.get_datacube(),
            self.test_maps[0],
            atol=1.e-6
            )

        dcube_f_cont = np.require(np.zeros_like(self.test_maps[0, 0]), 'F')
        mygridder = cygrid.WcsGrid(self.header, datacube=dcube_f_cont)
        mygridder.set_kernel(*self.kernel_args)

        with pytest.raises(TypeError):
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal
                )

    def test_user_datacube_memorycell(self):
        '''
        If user provides a data cube, it must be made sure that cygrid
        writes to the correct memory cell (even though, the
        `get_unweighted_datacube` method can have a different object id
        due to internal reshaping)
        '''

        dcube = np.zeros_like(self.test_maps[0, 0])  # float64
        mygridder = cygrid.WcsGrid(self.header, datacube=dcube)
        mygridder.set_kernel(*self.kernel_args)
        mygridder.grid(self.xcoords, self.ycoords, self.signal)

        assert_allclose(
            mygridder.get_datacube(),
            self.test_maps[0, 0],
            atol=1.e-6
            )
        assert_allclose(
            mygridder.get_unweighted_datacube(),
            dcube,
            atol=1.e-6
            )


class TestSlGrid:

    def setup(self):

        mapcenter = (131., 50.)
        mapsize = (1., 1.)  # degrees
        self.beamsize_fwhm = 0.05  # degrees

        avg_num_pixels_x = 5 * mapsize[0] / self.beamsize_fwhm
        avg_num_pixels_y = 5 * mapsize[1] / self.beamsize_fwhm

        scale = np.cos(np.radians(mapcenter[1]))

        with NumpyRNGContext(0):
            self.xcoords = np.random.uniform(
                mapcenter[0] - mapsize[0] / 2. / scale,
                mapcenter[0] + mapsize[0] / 2. / scale,
                int(avg_num_pixels_x * avg_num_pixels_y),
                ).astype(np.float64)
            self.ycoords = np.random.uniform(
                mapcenter[1] - mapsize[1] / 2.,
                mapcenter[1] + mapsize[1] / 2.,
                int(avg_num_pixels_x * avg_num_pixels_y),
                ).astype(np.float64)
            self.signal = np.random.normal(0., .01, len(self.xcoords))

            self.target_x = np.random.uniform(
                mapcenter[0] - mapsize[0] / 2. / scale,
                mapcenter[0] + mapsize[0] / 2. / scale,
                1000,
                ).astype(np.float64)
            self.target_y = np.random.uniform(
                mapcenter[1] - mapsize[1] / 2.,
                mapcenter[1] + mapsize[1] / 2.,
                1000,
                ).astype(np.float64)

        self.signal2 = np.column_stack([self.signal, self.signal ** 2])
        self.signal3 = np.column_stack([
            self.signal, self.signal ** 2,
            self.signal ** 3, self.signal ** 4
            ]).reshape((-1, 2, 2))

        kernelsize_fwhm = self.beamsize_fwhm / 2
        kernelsize_sigma = kernelsize_fwhm / 2.35
        support_radius = 3. * kernelsize_sigma
        hpx_min_res = kernelsize_sigma / 2.

        self.kernel_args = (
            'gauss1d',
            (0.5 / kernelsize_sigma ** 2,),
            support_radius,
            hpx_min_res,
            )

        self.test_sls = np.load(get_pkg_data_filename(
            'data/cygrid_test_sightlines.npy'
            ))

    def teardown(self):

        pass

    def test_gridding1d(self):

        mygridder = cygrid.SlGrid(self.target_x, self.target_y)
        mygridder.set_kernel(*self.kernel_args)

        mygridder.grid(self.xcoords, self.ycoords, self.signal)

        assert_allclose(
            mygridder.get_datacube(),
            self.test_sls[0, 0],
            atol=1.e-6
            )

    def test_gridding2d(self):

        mygridder = cygrid.SlGrid(self.target_x, self.target_y)
        mygridder.set_kernel(*self.kernel_args)

        mygridder.grid(self.xcoords, self.ycoords, self.signal2)

        assert_allclose(
            mygridder.get_datacube(),
            self.test_sls[0],
            atol=1.e-6
            )

    def test_gridding3d(self):

        mygridder = cygrid.SlGrid(self.target_x, self.target_y)
        mygridder.set_kernel(*self.kernel_args)

        mygridder.grid(self.xcoords, self.ycoords, self.signal3)

        if False:
            np.save(
                '/tmp/cygrid_test_sightlines.npy', mygridder.get_datacube()
                )

        assert_allclose(
            mygridder.get_datacube(),
            self.test_sls,
            atol=1.e-6
            )

    def test_kernel_not_set_error(self):

        mygridder = cygrid.SlGrid(self.target_x, self.target_y)

        with pytest.raises(RuntimeError):
            mygridder.grid(self.xcoords, self.ycoords, self.signal)

    def test_shape_error(self):
        '''
        Test for various shape errors
        '''

        mygridder = cygrid.SlGrid(self.target_x, self.target_y)
        mygridder.set_kernel(*self.kernel_args)

        with pytest.raises(cygrid.ShapeError):
            mygridder.grid(
                self.xcoords, self.ycoords[:, np.newaxis], self.signal
                )

        with pytest.raises(cygrid.ShapeError):
            # this should not work, as the second call to grid has
            # incompatible shape
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal
                )
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal[:, np.newaxis]
                )

        # now test with user-provided data cube
        dcube = np.zeros_like(self.test_sls[0])
        mygridder = cygrid.SlGrid(
            self.target_x, self.target_y, datacube=dcube
            )
        mygridder.set_kernel(*self.kernel_args)

        with pytest.raises(cygrid.ShapeError):
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal[:, np.newaxis]
                )

    def test_dtype_warning(self):
        '''
        Test for dtype warning

        In fact, at the moment there is only one possible warning, as
        everything else should be properly casted, etc.
        '''

        dcube = np.zeros_like(self.test_sls[0, 0])  # float32
        mygridder = cygrid.SlGrid(
            self.target_x, self.target_y,
            datacube=dcube, dtype=np.float64
            )
        mygridder.set_kernel(*self.kernel_args)

        with pytest.warns(UserWarning):
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal
                )

    def test_byteorder_warning(self):
        '''
        Test for byteorder warning

        Apparently astroquery.skyview can return wrong (non-native) byteorder.
        '''

        dcube = np.zeros_like(self.test_sls[0, 0])  # float32
        mygridder = cygrid.SlGrid(
            self.target_x, self.target_y,
            datacube=dcube, dtype=np.float64
            )
        mygridder.set_kernel(*self.kernel_args)

        signal_swapped = self.signal.byteswap().newbyteorder()

        with pytest.warns(UserWarning):
            mygridder.grid(
                self.xcoords, self.ycoords, signal_swapped
                )
    def test_c_contiguous(self):
        '''
        Cygrid should autocast to C-contiguous if necessary and raise
        an error if user-provided datacube is not C-contiguous
        '''

        signal2_f_cont = np.require(self.signal2, self.signal2.dtype, 'F')

        mygridder = cygrid.SlGrid(self.target_x, self.target_y)
        mygridder.set_kernel(*self.kernel_args)

        mygridder.grid(self.xcoords, self.ycoords, signal2_f_cont)

        assert_allclose(
            mygridder.get_datacube(),
            self.test_sls[0],
            atol=1.e-6
            )

        dcube_f_cont = np.require(np.zeros_like(self.test_sls[0, 0]), 'F')
        mygridder = cygrid.SlGrid(
            self.target_x, self.target_y, datacube=dcube_f_cont
            )
        mygridder.set_kernel(*self.kernel_args)

        with pytest.raises(TypeError):
            mygridder.grid(
                self.xcoords, self.ycoords, self.signal
                )

    def test_user_datacube_memorycell(self):
        '''
        If user provides a data cube, it must be made sure that cygrid
        writes to the correct memory cell (even though, the
        `get_unweighted_datacube` method can have a different object id
        due to internal reshaping)
        '''

        dcube = np.zeros_like(self.test_sls[0, 0])  # float64
        mygridder = cygrid.SlGrid(
            self.target_x, self.target_y, datacube=dcube
            )
        mygridder.set_kernel(*self.kernel_args)
        mygridder.grid(self.xcoords, self.ycoords, self.signal)

        assert_allclose(
            mygridder.get_datacube(),
            self.test_sls[0, 0],
            atol=1.e-6
            )
        assert_allclose(
            mygridder.get_unweighted_datacube(),
            dcube,
            atol=1.e-6
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np


__all__ = ['produce_mock_data']


def produce_mock_data(
        mapcenter, mapsize, beamsize_fwhm, num_samples, num_sources
        ):
    '''
    Produce mock raw data (including coordinate pairs) for testing.

    The raw data mimics a continuum observation (i.e., one flux density
    value per position) containing only Gaussian noise and some point
    sources.

    Parameters
    ----------
    mapcenter : tuple of floats
        Center coordinates, `(lon, lat)` of the generated (rectangular)
        map [deg].
    mapsize : tuple of floats
        Size of map, `(width, height)` of the generated map [deg].
    beamsize_fwhm : float
        FWHM beam size [deg].
        The artificial point sources, which are sampled, will be convolved
        with this.
    num_samples : int
        Number of samples to produce. Note that this must be large enough
        to ensure full sampling of the map (otherwise artifacts will
        appear, which are caused by so-called aliasing).
    num_sources : int
        Number of artificial point sources to be placed into the map.

    Returns
    -------
    lons : `~numpy.array` of float64 [1D]
        Longitude coordinates for each sample.
    lats : `~numpy.array` of float64 [1D]
        Latitude coordinates for each sample.
    signal : `~numpy.array` of float64 [1D]
        The artifical raw data signal for each position.

    Notes
    -----
    As in real astronomical measurements, the point sources are convolved
    with the instrument's response function (PSF), or telescope beam.
    The longitude map size is scaled with :math:`\\cos b`, where :math:`b`
    is the latitude. This is to ensure that the true-angular size roughly
    matches the given map width.
    '''

    lon_scale = np.cos(np.radians(mapcenter[1]))
    map_l, map_r = (
        mapcenter[0] - 1.1 * mapsize[0] / 2. / lon_scale,
        mapcenter[0] + 1.1 * mapsize[0] / 2. / lon_scale
        )
    map_b, map_t = (
        mapcenter[1] - 1.1 * mapsize[1] / 2.,
        mapcenter[1] + 1.1 * mapsize[1] / 2.,
        )

    # coordinates are drawn from a uniform distribution
    lons = np.random.uniform(map_l, map_r, num_samples).astype(np.float64)
    lats = np.random.uniform(map_b, map_t, num_samples).astype(np.float64)

    # add Gaussian noise
    signal = np.random.normal(0., 1., len(lons))

    beamsize_sigma = beamsize_fwhm / np.sqrt(8 * np.log(2))

    # put in artifical point source, with random amplitudes
    # we'll assume a Gaussian-shaped PSF

    def gauss2d(x, y, x0, y0, A, s):
        return A * np.exp(-((x - x0) ** 2 + (y - y0) ** 2) / 2. / s ** 2)

    for _ in range(num_sources):

        sou_x = np.random.uniform(map_l, map_r, 1).astype(np.float64)
        sou_y = np.random.uniform(map_b, map_t, 1).astype(np.float64)
        A = np.random.uniform(0, 10, 1)
        signal += gauss2d(lons, lats, sou_x, sou_y, A, beamsize_sigma)

    return lons, lats, signal



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Allow cythonizing of our pyx files and provide custom compiler options.
'''

import os
from setuptools.extension import Extension
import platform
# Note: importing numpy from here won't work, see:
# http://docs.astropy.org/en/stable/development/ccython.html#using-numpy-c-headers
# import numpy as np
# 'include_dirs': [np.get_include()], --> 'include_dirs': 'numpy'

PYXDIR = os.path.relpath(os.path.dirname(__file__))


def get_extensions():

    comp_args = {
        'extra_compile_args': ['-fopenmp', '-O3', '-std=c++11'],
        'extra_link_args': ['-fopenmp'],
        'language': 'c++',
        # 'libraries': ['m'],
        'include_dirs': ['numpy'],
        }

    if platform.system().lower() == 'windows':
        comp_args = {
            'extra_compile_args': ['/openmp'],
            'language': 'c++',
            'include_dirs': ['numpy'],
            }
    elif 'darwin' in platform.system().lower():
        # os.environ["CC"] = "g++-6"
        # os.environ["CPP"] = "cpp-6"
        # os.environ["CXX"] = "g++-6"
        # os.environ["LD"] = "gcc-6"
        from subprocess import getoutput

        extra_compile_args = [
            '-fopenmp', '-O3', '-mmacosx-version-min=10.7', '-std=c++11',
            ]
        # if ('clang' in getoutput('gcc -v')) and all(
        #         'command not found' in getoutput('gcc-{:d} -v'.format(d))
        #         for d in [6, 7, 8, 9]
        #         ):
        if 'clang' in getoutput('gcc -v'):
            extra_compile_args += ['-stdlib=libc++', ]
        comp_args = {
            'extra_compile_args': extra_compile_args,
            'extra_link_args': [
                '-fopenmp',  # '-lgomp'
                ],
            'language': 'c++',
            'include_dirs': ['numpy'],
            }

    ext_module_cygrid_cygrid = Extension(
        name='cygrid.cygrid',
        sources=[os.path.join(PYXDIR, 'cygrid.pyx')],
        **comp_args
        )

    ext_module_cygrid_helpers = Extension(
        name='cygrid.helpers',
        sources=[os.path.join(PYXDIR, 'helpers.pyx')],
        **comp_args
        )

    ext_module_cygrid_healpix = Extension(
        name='cygrid.healpix',
        sources=[os.path.join(PYXDIR, 'healpix.pyx')],
        **comp_args
        )

    ext_module_cygrid_hphashtab = Extension(
        name='cygrid.hphashtab',
        sources=[os.path.join(PYXDIR, 'hphashtab.pyx')],
        **comp_args
        )

    ext_module_cygrid_kernels = Extension(
        name='cygrid.kernels',
        sources=[os.path.join(PYXDIR, 'kernels.pyx')],
        **comp_args
        )

    # print('get_extensions', ext_module_cygrid_cygrid)
    return [
        ext_module_cygrid_cygrid,
        ext_module_cygrid_helpers,
        ext_module_cygrid_healpix,
        ext_module_cygrid_hphashtab,
        ext_module_cygrid_kernels
        ]
